@@ -18,6 +18,7 @@ from timepiece.lookups import ProjectLookup, QuickLookup
 from timepiece.lookups import UserLookup, BusinessLookup
 
 from timepiece.models import Project, Entry, Activity, UserProfile, Attribute
+from timepiece.models import ProjectHours
 from timepiece.fields import UserModelChoiceField
 from timepiece import models as timepiece
 from timepiece import utils
@@ -29,7 +30,7 @@ class ProjectFiltersForm(forms.Form):
         ('week', 'Week'),
         ('month', 'Month'),
     ]
-    DEFAULT_TRUNC = TRUNC_CHOICES[2][0]
+    DEFAULT_TRUNC = TRUNC_CHOICES[1][0]
     billable = forms.BooleanField(initial=True, required=False)
     non_billable = forms.BooleanField(label='Non-Billable', initial=True,
                                       required=False)
@@ -77,12 +78,12 @@ class EditPersonForm(auth_forms.UserChangeForm):
         label=_(u'Repeat Password'),
         widget=forms.PasswordInput(render_value=False))
 
-    class Meta:
-        model = auth_models.User
-        fields = (
-            "username", "first_name", "last_name",
-            "email", "is_active", "is_staff"
-        )
+    def __init__(self, *args, **kwargs):
+        super(EditPersonForm, self).__init__(*args, **kwargs)
+
+        # In 1.4 this field is created even if it is excluded in Meta.
+        if 'password' in self.fields:
+            del(self.fields['password'])
 
     def clean_password(self):
         return self.cleaned_data.get('password_one', None)
@@ -106,6 +107,11 @@ class EditPersonForm(auth_forms.UserChangeForm):
             instance.save()
         return instance
 
+    class Meta:
+        model = auth_models.User
+        fields = ('username', 'first_name', 'last_name', 'email', 'is_active',
+                'is_staff')
+
 
 class QuickSearchForm(forms.Form):
     quick_search = selectable_forms.AutoCompleteSelectField(
@@ -159,13 +165,14 @@ class AddUserToProjectForm(forms.Form):
 
 
 class ClockInForm(forms.ModelForm):
-    active_comment = forms.CharField(label='Notes for the current entry',
+    active_comment = forms.CharField(label='Notes for the active entry',
                                      widget=forms.Textarea, required=False)
 
     class Meta:
         model = timepiece.Entry
         fields = (
             'active_comment', 'location', 'project', 'activity', 'start_time',
+            'comments'
         )
 
     def __init__(self, *args, **kwargs):
@@ -357,8 +364,14 @@ STATUS_CHOICES.extend(timepiece.ENTRY_STATUS)
 
 
 class DateForm(forms.Form):
-    from_date = forms.DateField(label="From", required=False)
-    to_date = forms.DateField(label="To", required=False)
+    DATE_FORMAT = '%m/%d/%Y'
+
+    from_date = forms.DateField(label="From", required=False,
+        input_formats=(DATE_FORMAT,),
+        widget=forms.DateInput(format=DATE_FORMAT))
+    to_date = forms.DateField(label="To", required=False,
+        input_formats=(DATE_FORMAT,),
+        widget=forms.DateInput(format=DATE_FORMAT))
     status = forms.ChoiceField(choices=STATUS_CHOICES,
         widget=forms.HiddenInput(), required=False)
     activity = forms.ModelChoiceField(
@@ -382,9 +395,11 @@ class DateForm(forms.Form):
     def save(self):
         from_date = self.cleaned_data.get('from_date', '')
         to_date = self.cleaned_data.get('to_date', '')
-        if to_date:
-            to_date += timedelta(days=1)
-        return (from_date, to_date)
+        returned_date = to_date
+
+        if returned_date:
+            returned_date += timedelta(days=1)
+        return (from_date, returned_date)
 
 
 class YearMonthForm(forms.Form):
@@ -570,3 +585,73 @@ class DeleteForm(forms.Form):
             else:
                 return True
         return False
+
+
+class BillableHoursForm(forms.Form):
+    TRUNC_CHOICES = (
+        ('day', 'Day'),
+        ('week', 'Week'),
+        ('month', 'Month'),
+    )
+    trunc = forms.ChoiceField(
+        label='Group By',
+        choices=TRUNC_CHOICES,
+        widget=forms.RadioSelect(),
+        required=False,
+        initial=TRUNC_CHOICES[1][0])
+    people = forms.MultipleChoiceField(
+        required=False,
+        widget=forms.CheckboxSelectMultiple()
+    )
+    activities = forms.ModelMultipleChoiceField(
+        widget=forms.CheckboxSelectMultiple(),
+        queryset=timepiece.Activity.objects.all(),
+        required=False,
+        initial=timepiece.Activity.objects.all()
+    )
+    project_types = forms.ModelMultipleChoiceField(
+        widget=forms.CheckboxSelectMultiple(),
+        queryset=timepiece.Attribute.objects.all(),
+        required=False,
+        initial=timepiece.Attribute.objects.all()
+    )
+
+    def __init__(self, *args, **kwargs):
+        super(BillableHoursForm, self).__init__(*args, **kwargs)
+
+        people = []
+        users = timepiece.Entry.no_join.values('user', 'user__first_name',
+            'user__last_name').distinct().order_by('user__first_name',
+            'user__last_name')
+        for u in users:
+            name = ' '.join([u['user__first_name'], u['user__last_name']])
+            person = (u['user'], name,)
+            people.append(person)
+
+        self.fields['people'].choices = people
+        self.fields['people'].initial = [p[0] for p in people]
+
+    def save(self):
+        return {
+            'people': self.cleaned_data['people'],
+            'activities': self.cleaned_data['activities'],
+            'project_types': self.cleaned_data['project_types']
+        }
+
+
+class ProjectHoursSearchForm(forms.Form):
+    week_start = forms.DateField(label='Week of', required=False,
+            input_formats=('%Y-%m-%d',),
+            widget=forms.DateInput(format='%Y-%m-%d'))
+
+    def clean_week_start(self):
+        week_start = self.cleaned_data.get('week_start', None)
+        return utils.get_week_start(week_start, False) if week_start else None
+
+
+class ProjectHoursForm(forms.ModelForm):
+    def __init__(self, *args, **kwargs):
+        super(ProjectHoursForm, self).__init__(*args, **kwargs)
+
+    class Meta:
+        model = ProjectHours
