@@ -7,6 +7,11 @@ from django.core.urlresolvers import reverse
 from django.db.models import Sum
 from django.contrib.auth.models import User
 
+try:
+    from django.utils import timezone
+except ImportError:
+    from timepiece import timezone
+
 from timepiece import models as timepiece
 from timepiece import forms as timepiece_forms
 from timepiece.tests.base import TimepieceDataTestCase
@@ -30,7 +35,7 @@ class ProjectionTest(TimepieceDataTestCase):
             hours = 4
             minutes = 0
         if not start:
-            start = datetime.datetime.now()
+            start = timezone.now()
         elif not isinstance(start, datetime.datetime):
             start = datetime.datetime.combine(start, datetime.time())
         end = start + datetime.timedelta(hours=hours, minutes=minutes)
@@ -43,24 +48,26 @@ class ProjectionTest(TimepieceDataTestCase):
     def test_week_start(self):
         """ Test that all days Sun. through Sat. return the previous Monday"""
         monday = datetime.date(2011, 1, 10)
-        self.assertEqual(monday, utils.get_week_start(monday))
+        self.assertEqual(monday, utils.get_week_start(monday).date())
         sunday = datetime.date(2011, 1, 16)
-        self.assertEqual(monday, utils.get_week_start(sunday))
+        self.assertEqual(monday, utils.get_week_start(sunday).date())
         following_monday = datetime.date(2011, 1, 17)
         saturday = datetime.date(2011, 1, 22)
-        self.assertEqual(following_monday, utils.get_week_start(saturday))
+        self.assertEqual(following_monday,
+            utils.get_week_start(saturday).date())
 
     def  test_month_start(self):
         """ Test that any day returns the first day of the month"""
-        days = [datetime.date(2011, 1, 1),
-                datetime.date(2011, 1, 16),
-                datetime.date(2011, 1, 17),
-                datetime.date(2011, 1, 22),
-                datetime.date(2011, 1, 31),
-                ]
+        days = [
+            datetime.date(2011, 1, 1),
+            datetime.date(2011, 1, 16),
+            datetime.date(2011, 1, 17),
+            datetime.date(2011, 1, 22),
+            datetime.date(2011, 1, 31),
+        ]
+        date = utils.add_timezone(datetime.datetime(2011, 1, 1))
         for day in days:
-            self.assertEqual(utils.get_month_start(day),
-                             datetime.date(2011, 1, 1))
+            self.assertEqual(utils.get_month_start(day), date)
 
     def test_generate_dates(self):
         """ Test generation of full date ranges """
