@@ -3,8 +3,14 @@ from decimal import Decimal
 
 from django.core.urlresolvers import reverse
 
+try:
+    from django.utils import timezone
+except ImportError:
+    from timepiece import timezone
+
 from timepiece import models as timepiece
 from timepiece.tests.base import TimepieceDataTestCase
+from timepiece import utils
 
 
 class ProjectTestCase(TimepieceDataTestCase):
@@ -21,11 +27,11 @@ class ProjectTestCase(TimepieceDataTestCase):
 
     def make_entries(self):
         days = [
-                datetime.datetime(2011, 1, 1),
-                datetime.datetime(2011, 1, 28),
-                datetime.datetime(2011, 1, 31),
-                datetime.datetime(2011, 2, 1),
-                datetime.datetime.now(),
+            utils.add_timezone(datetime.datetime(2011, 1, 1)),
+            utils.add_timezone(datetime.datetime(2011, 1, 28)),
+            utils.add_timezone(datetime.datetime(2011, 1, 31)),
+            utils.add_timezone(datetime.datetime(2011, 2, 1)),
+            timezone.now(),
         ]
         self.log_time(project=self.p1, start=days[0], delta=(1, 0))
         self.log_time(project=self.p2, start=days[0], delta=(1, 0))
