@@ -1,13 +1,19 @@
 import datetime
-from dateutil import relativedelta
+from dateutil.relativedelta import relativedelta
 from decimal import Decimal
 from random import randint
+import json
 
 from django.conf import settings
 from django.core.urlresolvers import reverse
 from django.db.models import Q
 from django.contrib.auth.models import Permission
 
+try:
+    from django.utils import timezone
+except ImportError:
+    from timepiece import timezone
+
 from timepiece.tests.base import TimepieceDataTestCase
 
 from timepiece import models as timepiece
@@ -15,9 +21,9 @@ from timepiece import forms as timepiece_forms
 from timepiece import utils
 
 
-class TestHourlyReport(TimepieceDataTestCase):
+class ReportsHelperBase(TimepieceDataTestCase):
     def setUp(self):
-        super(TestHourlyReport, self).setUp()
+        super(ReportsHelperBase, self).setUp()
         self.sick = self.create_project()
         self.vacation = self.create_project()
         settings.TIMEPIECE_PROJECTS = {
@@ -32,19 +38,17 @@ class TestHourlyReport(TimepieceDataTestCase):
         self.p5 = self.create_project(billable=True, name='3')
         self.default_projects = [self.p1, self.p2, self.p3, self.p4, self.p5]
         self.default_dates = [
-                datetime.datetime(2011, 1, 3),
-                datetime.datetime(2011, 1, 4),
-                datetime.datetime(2011, 1, 10),
-                datetime.datetime(2011, 1, 16),
-                datetime.datetime(2011, 1, 17),
-                datetime.datetime(2011, 1, 18)
+            utils.add_timezone(datetime.datetime(2011, 1, 3)),
+            utils.add_timezone(datetime.datetime(2011, 1, 4)),
+            utils.add_timezone(datetime.datetime(2011, 1, 10)),
+            utils.add_timezone(datetime.datetime(2011, 1, 16)),
+            utils.add_timezone(datetime.datetime(2011, 1, 17)),
+            utils.add_timezone(datetime.datetime(2011, 1, 18)),
         ]
-        self.url = reverse('hourly_report')
 
     def make_entries(self, user=None, projects=None, dates=None,
-                     hours=1, minutes=0):
+                 hours=1, minutes=0):
         """Make several entries to help with reports tests"""
-
         if not user:
             user = self.user
         if not projects:
@@ -58,6 +62,8 @@ class TestHourlyReport(TimepieceDataTestCase):
 
     def bulk_entries(self, start=datetime.datetime(2011, 1, 2),
                    end=datetime.datetime(2011, 1, 4)):
+        start = utils.add_timezone(start)
+        end = utils.add_timezone(end)
         dates = utils.generate_dates(start, end, 'day')
         projects = [self.p1, self.p2, self.p2, self.p4, self.p5, self.sick]
         self.make_entries(projects=projects, dates=dates,
@@ -67,31 +73,41 @@ class TestHourlyReport(TimepieceDataTestCase):
 
     def check_generate_dates(self, start, end, trunc, dates):
         for index, day in enumerate(utils.generate_dates(start, end, trunc)):
-            self.assertEqual(day, dates[index])
+            if isinstance(day, datetime.datetime):
+                day = day.date()
+            self.assertEqual(day, dates[index].date())
+
+
+class TestHourlyReport(ReportsHelperBase):
+    def setUp(self):
+        super(TestHourlyReport, self).setUp()
+        self.url = reverse('hourly_report')
 
     def testGenerateMonths(self):
-        dates = [datetime.datetime(2011, month, 1) for month in xrange(1, 13)]
-        start = datetime.datetime(2011, 1, 1)
-        end = datetime.datetime(2011, 12, 1)
+        dates = [utils.add_timezone(datetime.datetime(2011, month, 1))
+            for month in xrange(1, 13)]
+        start = datetime.date(2011, 1, 1)
+        end = datetime.date(2011, 12, 1)
         self.check_generate_dates(start, end, 'month', dates)
 
     def testGenerateWeeks(self):
         dates = [
-            datetime.datetime(2010, 12, 27),
-            datetime.datetime(2011, 01, 03),
-            datetime.datetime(2011, 01, 10),
-            datetime.datetime(2011, 01, 17),
-            datetime.datetime(2011, 01, 24),
-            datetime.datetime(2011, 01, 31),
+            utils.add_timezone(datetime.datetime(2010, 12, 27)),
+            utils.add_timezone(datetime.datetime(2011, 01, 03)),
+            utils.add_timezone(datetime.datetime(2011, 01, 10)),
+            utils.add_timezone(datetime.datetime(2011, 01, 17)),
+            utils.add_timezone(datetime.datetime(2011, 01, 24)),
+            utils.add_timezone(datetime.datetime(2011, 01, 31)),
         ]
-        start = datetime.datetime(2011, 1, 1)
-        end = datetime.datetime(2011, 2, 1)
+        start = utils.add_timezone(datetime.datetime(2011, 1, 1))
+        end = utils.add_timezone(datetime.datetime(2011, 2, 1))
         self.check_generate_dates(start, end, 'week', dates)
 
     def testGenerateDays(self):
-        dates = [datetime.datetime(2011, 1, day) for day in xrange(1, 32)]
-        start = datetime.datetime(2011, 1, 1)
-        end = datetime.datetime(2011, 1, 31)
+        dates = [utils.add_timezone(datetime.datetime(2011, 1, day))
+            for day in xrange(1, 32)]
+        start = utils.add_timezone(datetime.datetime(2011, 1, 1))
+        end = utils.add_timezone(datetime.datetime(2011, 1, 31))
         self.check_generate_dates(start, end, 'day', dates)
 
     def check_truncs(self, trunc, billable, non_billable):
@@ -121,7 +137,7 @@ class TestHourlyReport(TimepieceDataTestCase):
             pj_totals = utils.project_totals(entries, date_headers, hour_type)
             pj_totals = list(pj_totals)
             rows = pj_totals[0][0]
-            hours = [hours for name, hours in rows]
+            hours = [hours for name, user_id, hours in rows]
             totals = pj_totals[0][1]
             return hours, totals
         else:
@@ -137,9 +153,9 @@ class TestHourlyReport(TimepieceDataTestCase):
                       user=self.user2)
 
     def testDailyTotal(self):
-        start = datetime.datetime(2011, 1, 1)
-        day2 = datetime.datetime(2011, 1, 2)
-        end = datetime.datetime(2011, 1, 3)
+        start = utils.add_timezone(datetime.datetime(2011, 1, 1))
+        day2 = utils.add_timezone(datetime.datetime(2011, 1, 2))
+        end = utils.add_timezone(datetime.datetime(2011, 1, 3))
         self.log_daily(start, day2, end)
         trunc = 'day'
         date_headers = utils.generate_dates(start, end, trunc)
@@ -152,9 +168,9 @@ class TestHourlyReport(TimepieceDataTestCase):
                          [Decimal('1.00'), Decimal('4.50'), Decimal('2.00')])
 
     def testBillableNonBillable(self):
-        start = datetime.datetime(2011, 1, 1)
-        day2 = datetime.datetime(2011, 1, 2)
-        end = datetime.datetime(2011, 1, 3)
+        start = utils.add_timezone(datetime.datetime(2011, 1, 1))
+        day2 = utils.add_timezone(datetime.datetime(2011, 1, 2))
+        end = utils.add_timezone(datetime.datetime(2011, 1, 3))
         self.log_daily(start, day2, end)
         trunc = 'day'
         billableQ = Q(project__type__billable=True)
@@ -172,8 +188,8 @@ class TestHourlyReport(TimepieceDataTestCase):
         self.assertEqual(list(pj_non_billable), list(pj_non_billable_q))
 
     def testWeeklyTotal(self):
-        start = datetime.datetime(2011, 1, 3)
-        end = datetime.datetime(2011, 1, 6)
+        start = utils.add_timezone(datetime.datetime(2011, 1, 3))
+        end = utils.add_timezone(datetime.datetime(2011, 1, 6))
         self.bulk_entries(start, end)
         trunc = 'week'
         date_headers = utils.generate_dates(start, end, trunc)
@@ -183,15 +199,15 @@ class TestHourlyReport(TimepieceDataTestCase):
         self.assertEqual(pj_totals[1], [72])
 
     def testMonthlyTotal(self):
-        start = datetime.datetime(2011, 1, 1)
-        end = datetime.datetime(2011, 3, 1)
+        start = utils.add_timezone(datetime.datetime(2011, 1, 1))
+        end = utils.add_timezone(datetime.datetime(2011, 3, 1))
         trunc = 'month'
         last_day = randint(5, 10)
         worked1 = randint(1, 3)
         worked2 = randint(1, 3)
         for month in xrange(1, 7):
             for day in xrange(1, last_day + 1):
-                day = datetime.datetime(2011, month, day)
+                day = utils.add_timezone(datetime.datetime(2011, month, day))
                 self.log_time(start=day, delta=(worked1, 0), user=self.user)
                 self.log_time(start=day, delta=(worked2, 0), user=self.user2)
         date_headers = utils.generate_dates(start, end, trunc)
@@ -203,6 +219,8 @@ class TestHourlyReport(TimepieceDataTestCase):
 
     def argsHelper(self, args={}, start=datetime.datetime(2011, 1, 2),
                    end=datetime.datetime(2011, 1, 4)):
+        start = utils.add_timezone(start)
+        end = utils.add_timezone(end)
         args.update({
             'from_date': start.strftime('%m/%d/%Y'),
             'to_date': end.strftime('%m/%d/%Y'),
@@ -307,8 +325,9 @@ class TestHourlyReport(TimepieceDataTestCase):
         self.checkTotals(args, data)
 
     def testForm_Month(self):
-        start = datetime.datetime(2011, 1, 4)
-        end = datetime.datetime(2011, 3, 28)
+        tz = timezone.get_current_timezone()
+        start = datetime.datetime(2011, 1, 4, tzinfo=tz)
+        end = datetime.datetime(2011, 3, 28, tzinfo=tz)
         args = {
             'billable': True,
             'non_billable': False,
@@ -405,3 +424,68 @@ class TestHourlyReport(TimepieceDataTestCase):
         self.user.save()
         response = self.client.get(self.url)
         self.assertEqual(response.status_code, 200)
+
+
+class TestBillableHours(ReportsHelperBase):
+    def setUp(self):
+        super(TestBillableHours, self).setUp()
+        self.url = reverse('billable_hours')
+        self.permission = Permission.objects.filter(
+            codename='view_entry_summary'
+        )
+        self.admin = self.create_user('admin', 'e@e.com', 'abc')
+        self.admin.user_permissions = self.permission
+        self.from_date = datetime.datetime(2011, 1, 2)
+        self.to_date = datetime.datetime(2011, 1, 4)
+        self.dates_data = ['12/27/2010', '01/03/2011']
+
+    def get_entries_data(self):
+        projects = getattr(settings, 'TIMEPIECE_PROJECTS', {})
+        # Account for the day added by the form
+        query = Q(end_time__gt=utils.get_week_start(self.from_date),
+            end_time__lt=self.to_date + relativedelta(days=1))
+        query &= ~Q(project__in=projects.values())
+        entries = timepiece.Entry.objects.date_trunc('week',
+            extra_values=('activity', 'project__status')).filter(query)
+        return entries
+
+    def test_access_permission(self):
+        """
+        You should be able to access the page if you have the
+        correct permissions
+        """
+        self.client.login(username='admin', password='abc')
+
+        response = self.client.get(self.url)
+        self.assertEqual(response.status_code, 200)
+
+    def test_access_no_permission(self):
+        """A regular should have no access to the page"""
+        self.client.login(username='user', password='abc')
+
+        response = self.client.get(self.url)
+        self.assertEqual(response.status_code, 302)
+
+    def test_response_data(self):
+        """Test that the data returned is correct"""
+        self.bulk_entries()
+        self.client.login(username='admin', password='abc')
+
+        response = self.client.get(self.url, data={
+            'from_date': self.from_date.strftime('%m/%d/%Y'),
+            'to_date': self.to_date.strftime('%m/%d/%Y')
+        })
+        self.assertEqual(response.status_code, 200)
+
+        entries_data = self.get_entries_data().order_by('user', 'date')
+        response_data = json.loads(response.context['data'])
+        dates = json.loads(response.context['dates'])
+
+        self.assertEqual(dates, self.dates_data)
+
+        for user, data in response_data.iteritems():
+            last_name = user.strip()
+            entries_total = sum([float(e['hours']) for e in
+                entries_data.filter(user__last_name=last_name)])
+            response_total = sum(e['total'] for e in data)
+            self.assertEqual(entries_total, response_total)
