@@ -11,6 +11,11 @@ from django.contrib.auth.models import User, Permission
 from django.core.exceptions import ValidationError
 from django.forms import model_to_dict
 
+try:
+    from django.utils import timezone
+except ImportError:
+    from timepiece import timezone
+
 from timepiece.tests.base import TimepieceDataTestCase
 
 from timepiece import models as timepiece
@@ -26,16 +31,16 @@ class EditableTest(TimepieceDataTestCase):
         self.entry = self.create_entry({
             'user': self.user,
             'project': self.project,
-            'start_time': datetime.datetime.now() - datetime.timedelta(days=6),
-            'end_time':  datetime.datetime.now() - datetime.timedelta(days=6),
+            'start_time': timezone.now() - datetime.timedelta(days=6),
+            'end_time':  timezone.now() - datetime.timedelta(days=6),
             'seconds_paused': 0,
             'status': 'verified',
         })
         self.entry2 = self.create_entry({
             'user': self.user,
             'project': self.project,
-            'start_time': datetime.datetime.now() - datetime.timedelta(days=2),
-            'end_time':  datetime.datetime.now() - datetime.timedelta(days=2),
+            'start_time': timezone.now() - datetime.timedelta(days=2),
+            'end_time':  timezone.now() - datetime.timedelta(days=2),
             'seconds_paused': 0,
             'status': 'unverified',
         })
@@ -85,7 +90,7 @@ class MyLedgerTest(TimepieceDataTestCase):
 
     def testEmptyHourlySummary(self):
         self.client.login(username='user', password='abc')
-        now = datetime.datetime.now()
+        now = timezone.now()
         empty_month = now + relativedelta(months=1)
         data = {
             'year': empty_month.year,
@@ -115,11 +120,11 @@ class MyLedgerTest(TimepieceDataTestCase):
         self.p4 = self.create_project(billable=True, name='4')
         self.p3 = self.create_project(billable=False, name='1')
         days = [
-                datetime.datetime(2011, 1, 1),
-                datetime.datetime(2011, 1, 28),
-                datetime.datetime(2011, 1, 31),
-                datetime.datetime(2011, 2, 1),
-                datetime.datetime.now(),
+            utils.add_timezone(datetime.datetime(2011, 1, 1)),
+            utils.add_timezone(datetime.datetime(2011, 1, 28)),
+            utils.add_timezone(datetime.datetime(2011, 1, 31)),
+            utils.add_timezone(datetime.datetime(2011, 2, 1)),
+            timezone.now(),
         ]
         self.log_time(project=self.p1, start=days[0], delta=(1, 0))
         self.log_time(project=self.p2, start=days[0], delta=(1, 0))
@@ -162,7 +167,7 @@ class ClockInTest(TimepieceDataTestCase):
     def setUp(self):
         super(ClockInTest, self).setUp()
         self.url = reverse('timepiece-clock-in')
-        self.now = datetime.datetime.now()
+        self.now = timezone.now()
         self.ten_min_ago = self.now - datetime.timedelta(minutes=10)
         self.clock_in_form = {
             'project': self.project.pk,
@@ -520,7 +525,7 @@ class AutoActivityTest(TimepieceDataTestCase):
         """The worker has several entries on a project. Use the most recent"""
         self.client.login(username='user', password='abc')
         for day in xrange(0, 10):
-            this_day = datetime.datetime(2011, 1, 1)
+            this_day = utils.add_timezone(datetime.datetime(2011, 1, 1))
             this_day += datetime.timedelta(days=day)
             activity = self.activity if day == 9 else self.devl_activity
             self.log_time(start=this_day, project=self.project,
@@ -535,7 +540,7 @@ class AutoActivityTest(TimepieceDataTestCase):
         project1 = self.project
         project2 = self.project2
         for day in xrange(0, 10):
-            this_day = datetime.datetime(2011, 1, 1)
+            this_day = utils.add_timezone(datetime.datetime(2011, 1, 1))
             this_day += datetime.timedelta(days=day)
             #Cycle through projects and activities
             project = project1 if day % 2 == 0 else project2
@@ -550,8 +555,8 @@ class ClockOutTest(TimepieceDataTestCase):
         super(ClockOutTest, self).setUp()
         self.client.login(username='user', password='abc')
         #create an open entry via clock in, so clock out tests don't have to
-        self.default_end_time = datetime.datetime.now()
-        back = datetime.datetime.now() - datetime.timedelta(hours=5)
+        self.default_end_time = timezone.now()
+        back = timezone.now() - datetime.timedelta(hours=5)
         entry = self.create_entry({
             'user': self.user,
             'start_time': back,
@@ -696,7 +701,7 @@ class ClockOutTest(TimepieceDataTestCase):
         existing entry
         """
         #Create a closed and valid entry
-        now = datetime.datetime.now() - datetime.timedelta(hours=5)
+        now = timezone.now() - datetime.timedelta(hours=5)
         entry1_data = ({
             'user': self.user,
             'project': self.project,
@@ -740,7 +745,7 @@ class CheckOverlap(TimepieceDataTestCase):
     def setUp(self):
         super(CheckOverlap, self).setUp()
         self.client.login(username='user', password='abc')
-        self.now = datetime.datetime.now()
+        self.now = timezone.now()
         #define start and end times to create valid entries
         self.start = self.now - datetime.timedelta(days=0, hours=8)
         self.end = self.now - datetime.timedelta(days=0)
@@ -808,12 +813,12 @@ class CreateEditEntry(TimepieceDataTestCase):
     def setUp(self):
         super(CreateEditEntry, self).setUp()
         self.client.login(username='user', password='abc')
-        self.now = datetime.datetime.now()
+        self.now = timezone.now()
         valid_start = self.now - datetime.timedelta(days=1)
         valid_end = valid_start + datetime.timedelta(hours=1)
-        two_hour_ago = self.now - datetime.timedelta(hours=2)
-        one_hour_ago = self.now - datetime.timedelta(hours=1)
-        ten_min_ago = self.now - datetime.timedelta(minutes=10)
+        self.ten_min_ago = self.now - datetime.timedelta(minutes=10)
+        self.two_hour_ago = self.now - datetime.timedelta(hours=2)
+        self.one_hour_ago = self.now - datetime.timedelta(hours=1)
         #establish data, entries, urls for all tests
         self.default_data = {
             'project': self.project.pk,
@@ -829,23 +834,23 @@ class CreateEditEntry(TimepieceDataTestCase):
             'user': self.user,
             'project': self.project,
             'activity': self.devl_activity,
-            'start_time': two_hour_ago,
-            'end_time': one_hour_ago,
+            'start_time': self.two_hour_ago,
+            'end_time': self.one_hour_ago,
         }
         self.current_entry_data = {
             'user': self.user,
             'project': self.project,
             'activity': self.devl_activity,
-            'start_time': ten_min_ago,
+            'start_time': self.ten_min_ago,
         }
         self.closed_entry = self.create_entry(self.closed_entry_data)
         self.current_entry = self.create_entry(self.current_entry_data)
         self.closed_entry_data.update({
-            'st_str': two_hour_ago.strftime('%H:%M:%S'),
-            'end_str': one_hour_ago.strftime('%H:%M:%S'),
+            'st_str': self.two_hour_ago.strftime('%H:%M:%S'),
+            'end_str': self.one_hour_ago.strftime('%H:%M:%S'),
         })
         self.current_entry_data.update({
-            'st_str': ten_min_ago.strftime('%H:%M:%S'),
+            'st_str': self.ten_min_ago.strftime('%H:%M:%S'),
         })
         self.create_url = reverse('timepiece-add')
         self.edit_closed_url = reverse('timepiece-update',
@@ -1015,12 +1020,115 @@ class CreateEditEntry(TimepieceDataTestCase):
         err_msg += 'choose among development, and Work'
         self.assertFormError(response, 'form', None, err_msg)
 
+    def add_entry_test_helper(self):
+        self.client.login(username='user', password='abc')
+
+        response = self.client.post(self.create_url, data=self.default_data,
+            follow=True)
+        self.assertEqual(response.status_code, 200)
+
+        msg = 'You cannot add/edit entries after a timesheet has been ' \
+            'approved or invoiced. Please correct the start and end times.'
+        self.assertEqual([msg], response.context['form'].non_field_errors())
+
+    def test_add_approved_entries(self):
+        """
+        If your entries have been verified and then approved, you should
+        not be able to add entries for that time period
+        """
+        entry = self.create_entry({
+            'start_time': self.ten_min_ago,
+            'end_time': self.ten_min_ago + datetime.timedelta(minutes=1)
+        })
+        entry.status = 'invoiced'
+        entry.save()
+
+        self.add_entry_test_helper()
+
+    def test_add_invoiced_entries(self):
+        """
+        If your entries have been verified, approved, and invoiced, you
+        should not be able to add entries for that time period
+        """
+        entry = self.create_entry({
+            'start_time': self.ten_min_ago,
+            'end_time': self.ten_min_ago + datetime.timedelta(minutes=1)
+        })
+        entry.status = 'invoiced'
+        entry.save()
+
+        self.add_entry_test_helper()
+
+    def edit_entry_helper(self, status='approved'):
+        """Helper function for editing approved entries"""
+        entry = self.create_entry({
+            'project': self.project,
+            'start_time': self.now - relativedelta(hours=6),
+            'end_time': self.now - relativedelta(hours=5),
+            'status': status
+        })
+        url = reverse('timepiece-update', args=(entry.pk,))
+
+        data = self.default_data
+        data.update({
+            'start_time_0': entry.start_time.strftime('%m/%d/%Y'),
+            'start_time_1': entry.start_time.strftime('%H:%M:%S'),
+            'end_time_0': entry.end_time.strftime('%m/%d/%Y'),
+            'end_time_1': entry.end_time.strftime('%H:%M:%S'),
+        })
+
+        return url, entry, data
+
+    def test_admin_edit_approved_entry(self):
+        """
+        An administrator (or anyone with view_payroll_summary perm) should
+        be able to edit an entry even if theyve been approved
+        """
+        self.client.logout()
+        self.client.login(username='superuser', password='abc')
+
+        url, entry, data = self.edit_entry_helper()
+
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+
+        response = self.client.post(url, data=data, follow=True)
+        self.assertEqual(response.status_code, 200)
+        self.assertContains(response,
+            'The entry has been updated successfully.')
+
+        self.assertEqual(self.user, entry.user)
+
+    def test_user_edit_approved_entry(self):
+        """A regular user shouldnt be able to edit an approved entry"""
+        url, entry, data = self.edit_entry_helper()
+
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 404)
+
+        response = self.client.post(url, data=data)
+        self.assertEqual(response.status_code, 404)
+
+    def test_edit_invoiced_entry(self):
+        """You shouldnt be able to edit an invoiced entry"""
+        self.client.logout()
+        self.client.login(username='superuser', password='abc')
+
+        url, entry, data = self.edit_entry_helper('invoiced')
+
+        response = self.client.post(url, data=data, follow=True)
+        self.assertEqual(response.status_code, 200)
+
+        msg = 'You cannot add/edit entries after a timesheet has been ' \
+            'approved or invoiced. Please correct the start and end times.'
+        self.assertContains(response, msg)
+
 
 class StatusTest(TimepieceDataTestCase):
     def setUp(self):
         super(StatusTest, self).setUp()
         self.client.login(username='user', password='abc')
-        self.now = datetime.datetime.now()
+        self.now = timezone.now()
         from_date = utils.get_month_start(self.now)
         self.from_date = from_date
         self.sheet_url = reverse('view_person_time_sheet',
@@ -1041,12 +1149,11 @@ class StatusTest(TimepieceDataTestCase):
         self.admin.save()
         self.client.login(username='admin', password='abc')
 
-    def login_with_permission(self):
+    def login_with_permissions(self, *codenames):
         """Helper to login as a user with correct permissions"""
-        view_entry_summary = Permission.objects.get(
-            codename=('view_entry_summary'))
+        perms = Permission.objects.filter(codename__in=codenames)
         self.perm_user = User.objects.create_user('perm', 'e@e.com', 'abc')
-        self.perm_user.user_permissions.add(view_entry_summary)
+        self.perm_user.user_permissions.add(*perms)
         self.perm_user.save()
         self.client.login(username='perm', password='abc')
 
@@ -1063,16 +1170,27 @@ class StatusTest(TimepieceDataTestCase):
         self.assertTrue(response.context['show_verify'])
         self.assertFalse(response.context['show_approve'])
 
-    def test_approve_link(self):
+    def test_approve_link_no_permission(self):
+        """Permission is required to see approve timesheet link."""
         entry = self.create_entry({
             'user': self.user,
             'start_time': self.now - datetime.timedelta(hours=1),
             'end_time': self.now,
             'status': 'verified'
         })
-
         response = self.client.get(self.sheet_url)
-        self.assertTrue(response.status_code)
+        self.assertFalse(response.context['show_approve'])
+
+    def test_approve_link(self):
+        self.login_with_permissions('view_entry_summary', 'approve_timesheet')
+        entry = self.create_entry({
+            'user': self.user,
+            'start_time': self.now - datetime.timedelta(hours=1),
+            'end_time': self.now,
+            'status': 'verified'
+        })
+        response = self.client.get(self.sheet_url)
+        self.assertEquals(response.status_code, 200)
 
         self.assertTrue(response.context['show_approve'])
         self.assertFalse(response.context['show_verify'])
@@ -1089,7 +1207,7 @@ class StatusTest(TimepieceDataTestCase):
         self.assertEquals(messages._loaded_messages[0].message, msg)
 
     def test_no_hours_approve(self):
-        self.login_with_permission()
+        self.login_with_permissions('approve_timesheet', 'view_entry_summary')
         response = self.client.get(self.approve_url, follow=True)
         self.assertEquals(response.status_code, 200)
 
@@ -1204,9 +1322,9 @@ class StatusTest(TimepieceDataTestCase):
         self.assertNotContains(response, self.verify_url)
         entry = self.create_entry(data={
             'user': self.user,
-            'start_time': datetime.datetime.now() - \
+            'start_time': timezone.now() - \
                 datetime.timedelta(hours=1),
-            'end_time':  datetime.datetime.now(),
+            'end_time':  timezone.now(),
         })
         response = self.client.get(self.sheet_url)
         self.assertTrue(response.context['show_verify'])
@@ -1221,9 +1339,8 @@ class StatusTest(TimepieceDataTestCase):
         self.assertFalse(response.context['show_approve'])
         entry = self.create_entry(data={
             'user': self.user,
-            'start_time': datetime.datetime.now() - \
-                datetime.timedelta(hours=1),
-            'end_time':  datetime.datetime.now(),
+            'start_time': timezone.now() - datetime.timedelta(hours=1),
+            'end_time':  timezone.now(),
         })
         response = self.client.get(self.sheet_url)
         self.assertFalse(response.context['show_approve'])
@@ -1239,9 +1356,9 @@ class StatusTest(TimepieceDataTestCase):
     def testVerifyPage(self):
         entry = self.create_entry(data={
             'user': self.user,
-            'start_time': datetime.datetime.now() - \
+            'start_time': timezone.now() - \
                 datetime.timedelta(hours=1),
-            'end_time':  datetime.datetime.now(),
+            'end_time':  timezone.now(),
         })
         response = self.client.get(self.verify_url)
         entries = self.user.timepiece_entries.all()
@@ -1250,12 +1367,11 @@ class StatusTest(TimepieceDataTestCase):
         self.assertEquals(entries[0].status, 'verified')
 
     def testApprovePage(self):
-        self.login_with_permission()
+        self.login_with_permissions('approve_timesheet', 'view_entry_summary')
         entry = self.create_entry(data={
             'user': self.user,
-            'start_time': datetime.datetime.now() - \
-                datetime.timedelta(hours=1),
-            'end_time':  datetime.datetime.now(),
+            'start_time': timezone.now() - datetime.timedelta(hours=1),
+            'end_time':  timezone.now(),
         })
 
         self.assertEquals(entry.status, 'unverified')
@@ -1273,7 +1389,7 @@ class StatusTest(TimepieceDataTestCase):
         """A regular user should not be able to reject an entry"""
         self.client.login(username='user', password='abc')
 
-        now = datetime.datetime.now()
+        now = timezone.now()
         entry = self.create_entry({
             'user': self.user,
             'start_time': now - datetime.timedelta(hours=1),
@@ -1292,7 +1408,7 @@ class StatusTest(TimepieceDataTestCase):
         """
         self.client.login(username='user2', password='abc')
 
-        now = datetime.datetime.now()
+        now = timezone.now()
         entry = self.create_entry({
             'user': self.user,
             'start_time': now - datetime.timedelta(hours=1),
@@ -1308,9 +1424,9 @@ class StatusTest(TimepieceDataTestCase):
         self.login_as_admin()
         entry = self.create_entry(data={
             'user': self.user,
-            'start_time': datetime.datetime.now() - \
+            'start_time': timezone.now() - \
                 datetime.timedelta(hours=1),
-            'end_time':  datetime.datetime.now(),
+            'end_time':  timezone.now(),
         })
         reject_url = self.get_reject_url(entry.id)
 
@@ -1332,9 +1448,9 @@ class StatusTest(TimepieceDataTestCase):
     def testNotAllowedToRejectTimesheet(self):
         entry = self.create_entry(data={
             'user': self.user,
-            'start_time': datetime.datetime.now() - \
+            'start_time': timezone.now() - \
                 datetime.timedelta(hours=1),
-            'end_time':  datetime.datetime.now(),
+            'end_time':  timezone.now(),
         })
         reject_url = self.get_reject_url(entry.id)
         response = self.client.get(reject_url)
@@ -1362,16 +1478,16 @@ class TestTotals(TimepieceDataTestCase):
     def testGroupedTotals(self):
         self.client.login(username='user', password='abc')
         days = [
-                datetime.datetime(2010, 12, 20),
-                datetime.datetime(2010, 12, 27),
-                datetime.datetime(2010, 12, 28),
-                datetime.datetime(2011, 1, 3),
-                datetime.datetime(2011, 1, 4),
-                datetime.datetime(2011, 1, 10),
-                datetime.datetime(2011, 1, 16),
-                datetime.datetime(2011, 1, 17),
-                datetime.datetime(2011, 1, 18),
-                datetime.datetime(2011, 2, 2),
+            utils.add_timezone(datetime.datetime(2010, 12, 20)),
+            utils.add_timezone(datetime.datetime(2010, 12, 27)),
+            utils.add_timezone(datetime.datetime(2010, 12, 28)),
+            utils.add_timezone(datetime.datetime(2011, 1, 3)),
+            utils.add_timezone(datetime.datetime(2011, 1, 4)),
+            utils.add_timezone(datetime.datetime(2011, 1, 10)),
+            utils.add_timezone(datetime.datetime(2011, 1, 16)),
+            utils.add_timezone(datetime.datetime(2011, 1, 17)),
+            utils.add_timezone(datetime.datetime(2011, 1, 18)),
+            utils.add_timezone(datetime.datetime(2011, 2, 2))
         ]
         # Each week has two days of entries, except 12-20, and 2-2 but these
         # are excluded in the timespan queryset
@@ -1382,7 +1498,8 @@ class TestTotals(TimepieceDataTestCase):
                 self.log_time(project=self.p2, start=day, delta=(1, 0))
             else:
                 self.log_time(project=self.p3, start=day, delta=(1, 0))
-        from_date = utils.get_month_start(datetime.datetime(2011, 1, 19))
+        date = utils.add_timezone(datetime.datetime(2011, 1, 19))
+        from_date = utils.get_month_start(date)
         to_date = from_date + relativedelta(months=1)
         first_week = utils.get_week_start(from_date)
         entries = timepiece.Entry.objects.timespan(first_week, to_date=to_date)
@@ -1417,7 +1534,7 @@ class TestTotals(TimepieceDataTestCase):
 class HourlySummaryTest(TimepieceDataTestCase):
     def setUp(self):
         super(HourlySummaryTest, self).setUp()
-        self.now = datetime.datetime.now()
+        self.now = timezone.now()
         self.month = self.now.replace(day=1)
         self.url = reverse('view_person_time_sheet', args=(self.user.pk,))
         self.client.login(username='user', password='abc')
@@ -1481,8 +1598,12 @@ class HourlySummaryTest(TimepieceDataTestCase):
         This occurs in April 2012, so we are using that month
         as the basis for out test case
         """
-        april = datetime.datetime(month=4, day=1, year=2012)
-        march = datetime.datetime(month=3, day=26, year=2012)
+        april = utils.add_timezone(
+            datetime.datetime(month=4, day=1, year=2012)
+        )
+        march = utils.add_timezone(
+            datetime.datetime(month=3, day=26, year=2012)
+        )
         self.create_entry({
             'user': self.user,
             'start_time': april,
@@ -1504,7 +1625,114 @@ class HourlySummaryTest(TimepieceDataTestCase):
         ))
         self.assertEquals(response.status_code, 200)
         # entries context object is a ValuesQuerySet
+        extra_values = ('start_time', 'end_time', 'comments', 'seconds_paused',
+                'id', 'location__name', 'project__name', 'activity__name',
+                'status')
         entries = timepiece.Entry.objects \
             .timespan(april, span='month') \
-            .date_trunc('month', True)
+            .date_trunc('month', extra_values)
         self.assertEquals(list(entries), list(response.context['entries']))
+
+
+class MonthlyRejectTestCase(TimepieceDataTestCase):
+    def setUp(self):
+        super(MonthlyRejectTestCase, self).setUp()
+        self.now = timezone.now()
+        self.data = {
+            'month': self.now.month,
+            'year': self.now.year,
+            'yes': 'Yes'
+        }
+        self.url = reverse('timepiece-reject-entries', args=(self.user.pk,))
+
+    def create_entries(self, date, status):
+        """Create entries using a date and with a given status"""
+        self.create_entry({
+            'start_time': date,
+            'end_time': date + relativedelta(hours=1),
+            'status': status
+        })
+        self.create_entry({
+            'start_time': date + relativedelta(hours=2),
+            'end_time': date + relativedelta(hours=3),
+            'status': status
+        })
+
+    def test_page_permissions(self):
+        """
+        An admin should have the permission to reject a users entries
+        and unverify them
+        """
+        self.client.login(username='superuser', password='abc')
+        self.create_entries(self.now, 'verified')
+
+        response = self.client.get(self.url, data=self.data)
+        self.assertEqual(response.status_code, 200)
+
+        response = self.client.post(self.url, data=self.data)
+
+        entries = timepiece.Entry.no_join.filter(status='verified')
+        self.assertEquals(entries.count(), 0)
+
+    def test_page_no_permissions(self):
+        """
+        A regular user should not have the permissions to
+        get or post to the page
+        """
+        self.client.login(username='user', password='abc')
+        self.create_entries(timezone.now(), 'verified')
+
+        response = self.client.get(self.url, data=self.data)
+        self.assertEqual(response.status_code, 302)
+
+        response = self.client.post(self.url, data=self.data)
+
+        entries = timepiece.Entry.no_join.filter(status='verified')
+        self.assertEquals(entries.count(), 2)
+
+    def test_reject_entries_no_date(self):
+        """
+        If you are missing the month/year used to filter the entries
+        then the reject page should not show
+        """
+        self.client.login(username='superuser', password='abc')
+        self.create_entries(timezone.now(), 'verified')
+
+        data = {
+            'month': self.now.month
+        }
+        response = self.client.get(self.url, data=data)
+        self.assertEqual(response.status_code, 302)
+
+        data = {
+            'year': self.now.year
+        }
+        response = self.client.get(self.url, data=data)
+        self.assertEqual(response.status_code, 302)
+
+    def test_reject_entries_no_confirm(self):
+        """
+        If a post request contains the month/year but is missing the key
+        'yes', then the entries are not rejected
+        """
+        self.client.login(username='superuser', password='abc')
+        self.create_entries(timezone.now(), 'verified')
+
+        data = self.data
+        data.pop('yes')
+
+        response = self.client.post(self.url, data=data)
+
+        entries = timepiece.Entry.no_join.filter(status='verified')
+        self.assertEquals(entries.count(), 2)
+
+    def test_reject_approved_invoiced_entries(self):
+        """Entries that are approved invoiced should not be rejected"""
+        self.client.login(username='superuser', password='abc')
+        self.create_entries(timezone.now(), 'approved')
+        self.create_entries(timezone.now(), 'invoiced')
+
+        response = self.client.post(self.url, data=self.data)
+
+        entries = timepiece.Entry.no_join.filter(status='unverified')
+        self.assertEquals(entries.count(), 0)
