@@ -1,4 +1,8 @@
-from django.conf.urls.defaults import *
+try:
+    from django.conf.urls import patterns, include, url
+except ImportError:
+    from django.conf.urls.defaults import patterns, include, url
+
 from timepiece.models import Entry
 from timepiece import views
 
@@ -104,21 +108,45 @@ urlpatterns = patterns('',
     ### time sheets ###
 
     # Reports
-    url(r'^reports/$', views.hourly_report, name='hourly_report'),
-    url(r'^reports/summary/$', views.summary, name='timepiece-summary'),
-    url(r'^reports/payroll/$', views.payroll_summary, name='payroll_summary',),
+    url(
+        r'^reports/$',
+        views.HourlyReport.as_view(),
+        name='hourly_report',
+    ),
+    url(
+        r'^reports/summary/$',
+        views.summary,
+        name='timepiece-summary',
+    ),
+    url(
+        r'^reports/payroll/$',
+        views.payroll_summary,
+        name='payroll_summary',
+    ),
+    url(
+        r'^reports/billable/$',
+        views.BillableHours.as_view(),
+        name='billable_hours',
+    ),
+
     # People
     url(
         r'time-sheet/people/(?P<user_id>\d+)/$',
         views.view_person_time_sheet,
         name='view_person_time_sheet',
     ),
+    url(
+        r'^time-sheet/reject/(?P<user_id>\d+)/$',
+        views.reject_entries,
+        name='timepiece-reject-entries',
+    ),
     url(
         r'^time-sheet/(?P<action>verify|approve)/(?P<user_id>\d+)/' +
         r'(?P<from_date>\d\d\d\d-\d\d-\d\d)/$',
         views.change_person_time_sheet,
         name='change_person_time_sheet',
     ),
+
     # Projects
     url(
         r'^time-sheet/project/(?P<pk>\d+)/$',
@@ -200,4 +228,27 @@ urlpatterns = patterns('',
         views.ContractList.as_view(),
         name='list_contracts',
     ),
+    # project hours
+    url(
+        r'^schedule/$',
+        views.ProjectHoursView.as_view(),
+        name='project_hours',
+    ),
+    url(
+        r'^schedule/edit/$',
+        views.EditProjectHoursView.as_view(),
+        name='edit_project_hours',
+    ),
+
+    # ajax views
+    url(
+        r'^ajax/hours/$',
+        views.ProjectHoursAjaxView.as_view(),
+        name='project_hours_ajax_view',
+    ),
+    url(
+        r'^ajax/hours/(?P<pk>\d+)/$',
+        views.ProjectHoursDetailView.as_view(),
+        name='project_hours_detail_view',
+    ),
 )
