@@ -1,13 +1,16 @@
-import urllib
+import calendar
 import csv
 import datetime
-import calendar
 import math
 import urllib
+import json
+import urlparse
+from copy import deepcopy
 
 from decimal import Decimal
 from dateutil.relativedelta import relativedelta
 from dateutil import rrule
+from itertools import groupby
 
 from django.contrib import messages
 from django.template import RequestContext
@@ -21,19 +24,28 @@ from django.contrib.auth.models import User
 from django.contrib.auth import models as auth_models
 from django.db.models import Sum, Count, Q, F
 from django.db import transaction
+from django.db import DatabaseError
 from django.conf import settings
 from django.utils.datastructures import SortedDict
 from django.views.decorators.csrf import csrf_exempt
 from django.views.generic.base import TemplateView
 from django.views.generic import UpdateView, ListView, DetailView, View
 from django.utils.decorators import method_decorator
+from django.core import serializers, exceptions
+from django.contrib.contenttypes.models import ContentType
 
-from timepiece.utils import render_with, reverse_lazy
+try:
+    from django.utils import timezone
+except ImportError:
+    from timepiece import timezone
+
+from timepiece.utils import render_with, reverse_lazy, get_week_start
 
 from timepiece import models as timepiece
 from timepiece import utils
 from timepiece import forms as timepiece_forms
 from timepiece.templatetags.timepiece_tags import seconds_to_hours
+from timepiece.templatetags.timepiece_tags import get_active_hours
 
 
 @login_required
@@ -75,11 +87,12 @@ def view_entries(request):
     if request.user.has_perm('timepiece.can_clock_in'):
         view_entries = True
     week_start = utils.get_week_start()
+    time_q = Q(end_time__gte=week_start) | Q(end_time__isnull=True)
     entries = timepiece.Entry.objects.select_related(
         'project__business',
     ).filter(
-        user=request.user,
-        end_time__gte=week_start,
+        time_q,
+        user=request.user
     ).select_related('project', 'activity', 'location')
     today = datetime.date.today()
     assignments = timepiece.ContractAssignment.objects.filter(
@@ -89,10 +102,9 @@ def view_entries(request):
         contract__status='current',
     ).order_by('contract__project__type', 'end_date')
     assignments = assignments.select_related('user', 'contract__project__type')
-    activity_entries = entries.values(
+    activity_entries = list(entries.values(
         'billable',
-    ).annotate(sum=Sum('hours')).order_by('-sum')
-    current_total = entries.aggregate(sum=Sum('hours'))['sum']
+    ).annotate(sum=Sum('hours')).order_by('-sum'))
     others_active_entries = timepiece.Entry.objects.filter(
         end_time__isnull=True,
     ).exclude(
@@ -106,6 +118,14 @@ def view_entries(request):
         user=request.user,
         end_time__isnull=True,
     )
+
+    for current_entry in my_active_entries:
+        for activity_entry in activity_entries:
+            if current_entry.billable == activity_entry['billable']:
+                activity_entry['sum'] += get_active_hours(current_entry)
+                break
+    current_total = sum([entry['sum'] for entry in activity_entries])
+
 #     temporarily disabled until the allocations represent accurate goals
 #     -TM 6/27
     allocations = []
@@ -118,13 +138,16 @@ def view_entries(request):
 
     project_entries = entries.exclude(
         project__in=allocated_projects,
+        end_time__isnull=True
     ).values(
         'project__name', 'project__pk'
     ).annotate(sum=Sum('hours')).order_by('project__name')
     schedule = timepiece.PersonSchedule.objects.filter(
                                     user=request.user)
+    this_weeks_entries = entries.order_by('-start_time'). \
+        filter(end_time__gte=week_start)
     context = {
-        'this_weeks_entries': entries.order_by('-start_time'),
+        'this_weeks_entries': this_weeks_entries,
         'assignments': assignments,
         'allocations': allocations,
         'schedule': schedule,
@@ -228,7 +251,7 @@ def toggle_paused(request, entry_id):
         else:
             action = 'resumed'
 
-        delta = datetime.datetime.now() - entry.start_time
+        delta = timezone.now() - entry.start_time
         seconds = delta.seconds - entry.seconds_paused
         seconds += delta.days * 86400
 
@@ -255,9 +278,9 @@ def create_edit_entry(request, entry_id=None):
         try:
             entry = timepiece.Entry.no_join.get(
                 pk=entry_id,
-                user=request.user,
             )
-            if not entry.is_editable:
+            if not (entry.is_editable or
+                    request.user.has_perm('timepiece.view_payroll_summary')):
                 raise Http404
 
         except timepiece.Entry.DoesNotExist:
@@ -270,7 +293,7 @@ def create_edit_entry(request, entry_id=None):
         form = timepiece_forms.AddUpdateEntryForm(
             request.POST,
             instance=entry,
-            user=request.user,
+            user=entry.user if entry else request.user,
         )
         if form.is_valid():
             entry = form.save()
@@ -330,6 +353,43 @@ def reject_entry(request, entry_id):
                               context_instance=RequestContext(request))
 
 
+@permission_required('timepiece.view_payroll_summary')
+def reject_entries(request, user_id):
+    """
+    This allows admins to reject all entries, instead of just one
+    """
+    form = timepiece_forms.YearMonthForm(request.GET
+        or request.POST)
+    user = auth_models.User.objects.get(pk=user_id)
+    if form.is_valid():
+        from_date, to_date = form.save()
+        entries = timepiece.Entry.no_join.filter(status='verified', user=user,
+            start_time__gte=from_date, end_time__lte=to_date)
+        if request.POST.get('yes'):
+            if entries.exists():
+                count = entries.count()
+                entries.update(status='unverified')
+                msg = 'You have rejected %d previously verified entries.' \
+                    % count
+            else:
+                msg = 'There are no verified entries to reject.'
+            messages.info(request, msg)
+        else:
+            context = {
+                'date': from_date,
+                'timesheet_user': user
+            }
+            return render(request,
+                'timepiece/time-sheet/entry/reject_entries.html', context)
+    else:
+        msg = 'You must provide a month and year for entries to be rejected.'
+        messages.error(request, msg)
+
+    return HttpResponseRedirect(reverse('view_person_time_sheet',
+        args=(user_id,))
+    )
+
+
 @permission_required('timepiece.delete_entry')
 def delete_entry(request, entry_id):
     """
@@ -368,13 +428,18 @@ def delete_entry(request, entry_id):
 @permission_required('timepiece.view_entry_summary')
 @render_with('timepiece/time-sheet/reports/general_ledger.html')
 def summary(request, username=None):
-    if request.GET:
-        form = timepiece_forms.DateForm(request.GET)
-        if form.is_valid():
-            from_date, to_date = form.save()
-    else:
-        form = timepiece_forms.DateForm()
-        from_date, to_date = None, None
+    date = timezone.now() - relativedelta(months=1)
+    from_date = utils.get_month_start(date).date()
+    to_date = from_date + relativedelta(months=1)
+
+    form = timepiece_forms.YearMonthForm(request.GET or None, initial={
+        'month': from_date.month,
+        'year': from_date.year
+    })
+
+    if form.is_valid():
+        from_date, to_date = form.save()
+
     entries = timepiece.Entry.no_join.values(
         'project__id',
         'project__business__id',
@@ -403,6 +468,7 @@ def summary(request, username=None):
         'project_totals': project_totals,
         'total_hours': total_hours,
         'people_totals': people_totals,
+        'from_date': from_date
     }
     return context
 
@@ -434,15 +500,18 @@ class ProjectTimesheet(DetailView):
         if self.request.GET and year_month_form.is_valid():
             from_date, to_date = year_month_form.save()
         else:
-            from_date = utils.get_month_start(datetime.datetime.today()).date()
+            date = utils.add_timezone(datetime.datetime.today())
+            from_date = utils.get_month_start(date).date()
             to_date = from_date + relativedelta(months=1)
         entries_qs = timepiece.Entry.objects
         entries_qs = entries_qs.timespan(from_date, span='month').filter(
             project=project
         )
-        month_entries = entries_qs.date_trunc('month', True).order_by(
-            'start_time'
-        )
+        extra_values = ('start_time', 'end_time', 'comments', 'seconds_paused',
+                'id', 'location__name', 'project__name', 'activity__name',
+                'status')
+        month_entries = entries_qs.date_trunc('month',
+                extra_values).order_by('start_time')
         total = entries_qs.aggregate(hours=Sum('hours'))['hours']
         user_entries = entries_qs.order_by().values(
             'user__first_name', 'user__last_name').annotate(
@@ -507,7 +576,7 @@ def view_person_time_sheet(request, user_id):
     if not (request.user.has_perm('timepiece.view_entry_summary') or \
         user.pk == request.user.pk):
         return HttpResponseForbidden('Forbidden')
-    today_reset = datetime.datetime.today()
+    today_reset = utils.add_timezone(datetime.datetime.today())
     today_reset = today_reset.replace(hour=0, minute=0, second=0, \
         microsecond=0)
     from_date = utils.get_month_start(today_reset)
@@ -536,7 +605,10 @@ def view_person_time_sheet(request, user_id):
             from_date, to_date = year_month_form.save()
     entries_qs = timepiece.Entry.objects.filter(user=user)
     month_qs = entries_qs.timespan(from_date, span='month')
-    month_entries = month_qs.date_trunc('month', True)
+    extra_values = ('start_time', 'end_time', 'comments', 'seconds_paused',
+            'id', 'location__name', 'project__name', 'activity__name',
+            'status')
+    month_entries = month_qs.date_trunc('month', extra_values)
     # For grouped entries, back date up to the start of the week.
     first_week = utils.get_week_start(from_date)
     month_week = first_week + datetime.timedelta(weeks=1)
@@ -555,14 +627,16 @@ def view_person_time_sheet(request, user_id):
     summary = timepiece.Entry.summary(user, from_date, to_date)
     show_approve = show_verify = False
     if request.user.has_perm('timepiece.change_entry') or \
+        request.user.has_perm('timepiece.approve_timesheet') or \
         user == request.user:
         statuses = list(month_qs.values_list('status', flat=True))
         total_statuses = len(statuses)
         unverified_count = statuses.count('unverified')
         verified_count = statuses.count('verified')
         approved_count = statuses.count('approved')
+    if request.user.has_perm('timepiece.change_entry') or user == request.user:
         show_verify = unverified_count != 0
-    if request.user.has_perm('timepiece.change_entry'):
+    if request.user.has_perm('timepiece.approve_timesheet'):
         show_approve = verified_count + approved_count == total_statuses \
         and verified_count > 0 and total_statuses != 0
     context = {
@@ -597,7 +671,8 @@ def change_person_time_sheet(request, action, user_id, from_date):
             'timesheet'.format(action))
 
     try:
-        from_date = datetime.datetime.strptime(from_date, '%Y-%m-%d')
+        from_date = utils.add_timezone(
+            datetime.datetime.strptime(from_date, '%Y-%m-%d'))
     except (ValueError, OverflowError):
         raise Http404
     to_date = from_date + relativedelta(months=1)
@@ -659,9 +734,11 @@ def confirm_invoice_project(request, project_id, to_date, from_date=None):
     if not request.user.has_perm('timepiece.generate_project_invoice'):
         return HttpResponseForbidden('Forbidden')
     try:
-        to_date = datetime.datetime.strptime(to_date, '%Y-%m-%d')
+        to_date = utils.add_timezone(
+            datetime.datetime.strptime(to_date, '%Y-%m-%d'))
         if from_date:
-            from_date = datetime.datetime.strptime(from_date, '%Y-%m-%d')
+            from_date = utils.add_timezone(
+                datetime.datetime.strptime(from_date, '%Y-%m-%d'))
         else:
             from_date = None
     except (ValueError, OverflowError):
@@ -707,7 +784,8 @@ def confirm_invoice_project(request, project_id, to_date, from_date=None):
 
 @permission_required('timepiece.change_entrygroup')
 def invoice_projects(request):
-    to_date = utils.get_month_start(datetime.datetime.today()).date()
+    date = utils.add_timezone(datetime.datetime.today())
+    to_date = utils.get_month_start(date).date()
     from_date = None
     defaults = {
         'to_date': (to_date - relativedelta(days=1)).strftime('%m/%d/%Y'),
@@ -724,6 +802,7 @@ def invoice_projects(request):
         'project__type__pk', 'project__type__label', 'project__name', 'hours',
         'project__pk', 'status', 'project__status__label'
     ).annotate(s=Sum('hours')).order_by('project__type__label',
+                                        'project__status__label',
                                         'project__name', 'status')
     return render_to_response(
         'timepiece/time-sheet/invoice/make_invoice.html', {
@@ -1179,12 +1258,15 @@ def create_edit_project(request, project_id=None):
 @permission_required('timepiece.view_payroll_summary')
 @render_with('timepiece/time-sheet/reports/summary.html')
 def payroll_summary(request):
-    year_month_form = timepiece_forms.YearMonthForm(request.GET or None)
-    if request.GET and year_month_form.is_valid():
+    date = timezone.now() - relativedelta(months=1)
+    from_date = utils.get_month_start(date).date()
+    to_date = from_date + relativedelta(months=1)
+
+    year_month_form = timepiece_forms.YearMonthForm(request.GET or None,
+        initial={'month': from_date.month, 'year': from_date.year})
+
+    if year_month_form.is_valid():
         from_date, to_date = year_month_form.save()
-    else:
-        from_date = utils.get_month_start(datetime.datetime.today()).date()
-        to_date = from_date + relativedelta(months=1)
     last_billable = utils.get_last_billable_day(from_date)
     projects = getattr(settings, 'TIMEPIECE_PROJECTS', {})
     weekQ = Q(end_time__gt=utils.get_week_start(from_date),
@@ -1201,10 +1283,10 @@ def payroll_summary(request):
     # Monthly totals
     leave = timepiece.Entry.objects.filter(monthQ, ~workQ
                                   ).values('user', 'hours', 'project__name')
-    month_entries = timepiece.Entry.objects.date_trunc('month')
+    extra_values = ('project__type__label',)
+    month_entries = timepiece.Entry.objects.date_trunc('month', extra_values)
     month_entries_valid = month_entries.filter(monthQ, statusQ, workQ)
-    monthly_totals = list(utils.payroll_totals(month_entries_valid, from_date,
-                                               leave))
+    labels, monthly_totals = utils.payroll_totals(month_entries_valid, leave)
     # Unapproved and unverified hours
     entries = timepiece.Entry.objects.filter(monthQ)
     user_values = ['user__pk', 'user__first_name', 'user__last_name']
@@ -1219,6 +1301,7 @@ def payroll_summary(request):
         'monthly_totals': monthly_totals,
         'unverified': unverified.values_list(*user_values).distinct(),
         'unapproved': unapproved.values_list(*user_values).distinct(),
+        'labels': labels,
     }
 
 
@@ -1275,66 +1358,6 @@ def edit_settings(request):
     return {'profile_form': profile_form, 'user_form': user_form}
 
 
-@permission_required('timepiece.view_entry_summary')
-@render_with('timepiece/time-sheet/reports/hourly.html')
-@utils.date_filter
-def hourly_report(request, date_form, from_date, to_date, status, activity):
-    if not from_date:
-        from_date = utils.get_month_start(datetime.datetime.today()).date()
-    if not to_date:
-        to_date = from_date + relativedelta(months=1)
-    header_to = to_date - relativedelta(days=1)
-    trunc = timepiece_forms.ProjectFiltersForm.DEFAULT_TRUNC
-    query = Q(end_time__gt=utils.get_week_start(from_date),
-              end_time__lt=to_date)
-    if 'ok' in request.GET or 'export' in request.GET:
-        form = timepiece_forms.ProjectFiltersForm(request.GET)
-        if form.is_valid():
-            trunc = form.cleaned_data['trunc']
-            if not form.cleaned_data['paid_leave']:
-                projects = getattr(settings, 'TIMEPIECE_PROJECTS', {})
-                query &= ~Q(project__in=projects.values())
-            if form.cleaned_data['pj_select']:
-                query &= Q(project__in=form.cleaned_data['pj_select'])
-    else:
-        form = timepiece_forms.ProjectFiltersForm()
-    hour_type = form.get_hour_type()
-    entries = timepiece.Entry.objects.date_trunc(trunc).filter(query)
-    date_headers = utils.generate_dates(from_date, header_to, by=trunc)
-    project_totals = utils.project_totals(entries, date_headers, hour_type,
-                                          total_column=True) if entries else ''
-    if not request.GET.get('export', False):
-        return {
-            'date_form': date_form,
-            'from_date': from_date,
-            'date_headers': date_headers,
-            'pj_filters': form,
-            'trunc': trunc,
-            'project_totals': project_totals,
-        }
-    else:
-        from_date_str = from_date.strftime('%m-%d')
-        to_date_str = to_date.strftime('%m-%d')
-        response = HttpResponse(mimetype='text/csv')
-        response['Content-Disposition'] = \
-            'attachment; filename="%s_hours_%s_to_%s_by_%s.csv"' % (
-            hour_type, from_date_str, to_date_str, trunc)
-        writer = csv.writer(response)
-        headers = ['Name']
-        headers.extend([date.strftime('%m/%d/%Y') for date in date_headers])
-        headers.append('Total')
-        writer.writerow(headers)
-        for rows, totals in project_totals:
-            for name, hours in rows:
-                data = [name]
-                data.extend(hours)
-                writer.writerow(data)
-            total = ['Totals']
-            total.extend(totals)
-            writer.writerow(total)
-        return response
-
-
 class ContractDetail(DetailView):
     template_name = 'timepiece/time-sheet/contract/view.html'
     model = timepiece.ProjectContract
@@ -1372,7 +1395,8 @@ class DeleteView(TemplateView):
     def dispatch(self, request, *args, **kwargs):
         for permission in self.permissions:
             if not request.user.has_perm(permission):
-                messages.info(request, 'You do not have permission to access that')
+                messages.info(request,
+                    'You do not have permission to access that')
                 return HttpResponseRedirect(reverse_lazy('timepiece-entries'))
         return super(DeleteView, self).dispatch(request, *args, **kwargs)
 
@@ -1418,3 +1442,461 @@ class DeleteProjectView(DeleteView):
     model = timepiece.Project
     url_name = 'list_projects'
     permissions = ('timepiece.add_project', 'timepiece.change_project',)
+
+
+class DecimalEncoder(json.JSONEncoder):
+    def default(self, obj):
+        if isinstance(obj, Decimal):
+            return float(obj)
+        return super(DecimalEncoder, self).default(obj)
+
+
+class ReportMixin(object):
+    @method_decorator(permission_required('timepiece.view_entry_summary'))
+    def dispatch(self, request, *args, **kwargs):
+        return super(ReportMixin, self).dispatch(request, *args, **kwargs)
+
+    def get_context_data(self, **kwargs):
+        context = super(ReportMixin, self).get_context_data(**kwargs)
+
+        end = utils.get_month_start(timezone.now())
+        from_date, to_date = utils.process_dates(self.request.GET,
+            end - relativedelta(months=1), end)
+
+        date_form = timepiece_forms.DateForm({
+            'from_date': from_date,
+            'to_date': to_date
+        })
+        if date_form.is_valid():
+            from_date, to_date = date_form.save()
+
+        header_to = to_date - relativedelta(days=1)
+        trunc = timepiece_forms.ProjectFiltersForm.DEFAULT_TRUNC
+        query = Q(end_time__gt=utils.get_week_start(from_date),
+                  end_time__lt=to_date)
+
+        project_form = timepiece_forms.ProjectFiltersForm(self.request.GET)
+
+        if project_form.is_valid():
+            trunc = project_form.cleaned_data['trunc']
+            if not project_form.cleaned_data['paid_leave']:
+                projects = getattr(settings, 'TIMEPIECE_PROJECTS', {})
+                query &= ~Q(project__in=projects.values())
+            if project_form.cleaned_data['pj_select']:
+                query &= Q(project__in=project_form.cleaned_data['pj_select'])
+
+        entries = timepiece.Entry.objects.date_trunc(trunc,
+            extra_values=('activity', 'project__status')).filter(query)
+        date_headers = utils.generate_dates(from_date, header_to, by=trunc)
+
+        context.update({
+            'date_form': date_form,
+            'from_date': from_date,
+            'to_date': to_date,
+            'date_headers': date_headers,
+            'trunc': trunc,
+            'entries': entries,
+            'pj_filters': project_form
+        })
+        return context
+
+
+class HourlyReport(ReportMixin, CSVMixin, TemplateView):
+    template_name = 'timepiece/time-sheet/reports/hourly.html'
+
+    def get(self, request, *args, **kwargs):
+        export = request.GET.get('export', False)
+        context = self.get_context_data()
+
+        kls = CSVMixin if export else TemplateView
+        return kls.render_to_response(self, context)
+
+    def get_filename(self, context):
+        request = self.request.GET.copy()
+        from_date = request.get('from_date')
+        to_date = request.get('to_date')
+        return 'hours_{0}_to_{1}_by_{2}.csv'.format(from_date, to_date,
+            context['trunc'])
+
+    def convert_context_to_csv(self, context):
+        "Convert the context dictionary into a CSV file"
+        content = []
+        date_headers = context['date_headers']
+
+        headers = ['Name']
+        headers.extend([date.strftime('%m/%d/%Y') for date in date_headers])
+        headers.append('Total')
+        content.append(headers)
+
+        for rows, totals in context['project_totals']:
+            for name, user_id, hours in rows:
+                data = [name]
+                data.extend(hours)
+                content.append(data)
+            total = ['Totals']
+            total.extend(totals)
+            content.append(total)
+
+        return content
+
+    def get_context_data(self, **kwargs):
+        context = super(HourlyReport, self).get_context_data(**kwargs)
+        entries = context['entries']
+        date_headers = context['date_headers']
+        hour_type = context['pj_filters'].get_hour_type()
+
+        project_totals = utils.project_totals(entries, date_headers, hour_type,
+            total_column=True) if entries else ''
+
+        context.update({
+            'project_totals': project_totals,
+        })
+
+        return context
+
+
+class BillableHours(ReportMixin, TemplateView):
+    template_name = 'timepiece/time-sheet/reports/billable_hours.html'
+
+    def get_hours_data(self, data, entries, date_headers):
+        default_users = timepiece.Entry.no_join.values('user__pk',)
+        users = data.get('people', '') or default_users
+
+        activities = data.get('activities', '') or \
+            timepiece.Activity.objects.values_list('pk', flat=True)
+
+        types = data.get('project_types', '') or \
+            timepiece.Attribute.objects.values_list('pk', flat=True)
+
+        filtered_entries = entries.filter(user__in=users,
+            activity__in=activities, project__status__in=types)
+
+        project_data = utils.project_totals(filtered_entries, date_headers,
+            total_column=False)
+
+        hours_data = {}
+        dates = []
+
+        for rows, totals in project_data:
+            for user, user_id, hours in rows:
+                hours_data[user] = []
+
+                for hour in hours:
+                    date = hour['day'].strftime('%m/%d/%Y')
+                    if date not in dates:
+                        dates.append(date)
+
+                    hours_data.get(user, []).append({
+                        'date': date,
+                        'billable': hour['billable'],
+                        'nonbillable': hour['nonbillable'],
+                        'total': hour['total'],
+                    })
+
+        return dates, hours_data
+
+    def get_context_data(self, **kwargs):
+        context = super(BillableHours, self).get_context_data(**kwargs)
+        entries = context['entries']
+        date_headers = context['date_headers']
+
+        form = timepiece_forms.BillableHoursForm(self.request.GET or None)
+
+        form_data = form.save() if form.is_valid() else {}
+        dates, hours_data = self.get_hours_data(form_data, entries,
+            date_headers)
+
+        context.update({
+            'billable_form': form,
+            'data': json.dumps(hours_data, cls=DecimalEncoder),
+            'dates': json.dumps(dates),
+        })
+
+        return context
+
+
+class ProjectHoursMixin(object):
+    permissions = None
+
+    def dispatch(self, request, *args, **kwargs):
+        for perm in self.permissions:
+            if not request.user.has_perm(perm):
+                return HttpResponseRedirect(reverse('auth_login'))
+
+        # Since we use get param in multiple places, attach it to the class
+        default_week = utils.get_week_start(datetime.date.today()).date()
+
+        if request.method == 'GET':
+            week_start_str = request.GET.get('week_start', '')
+        else:
+            week_start_str = request.POST.get('week_start', '')
+
+        # Account for an empty string
+        self.week_start = default_week if week_start_str == '' \
+            else utils.get_week_start(datetime.datetime.strptime(
+                week_start_str, '%Y-%m-%d').date())
+
+        return super(ProjectHoursMixin, self).dispatch(request, *args,
+            **kwargs)
+
+    def get_hours_for_week(self, start=None):
+        week_start = start if start else self.week_start
+        week_end = week_start + relativedelta(days=7)
+
+        return timepiece.ProjectHours.objects.filter(
+            week_start__gte=week_start, week_start__lt=week_end)
+
+
+class ProjectHoursView(ProjectHoursMixin, TemplateView):
+    template_name = 'timepiece/hours/list.html'
+    permissions = ('timepiece.can_clock_in',)
+
+    def get_context_data(self, **kwargs):
+        context = super(ProjectHoursView, self).get_context_data(**kwargs)
+
+        form = timepiece_forms.ProjectHoursSearchForm(initial={
+            'week_start': self.week_start
+        })
+
+        project_hours = utils.get_project_hours_for_week(self.week_start) \
+            .filter(published=True)
+        people = utils.get_people_from_project_hours(project_hours)
+        id_list = [person[0] for person in people]
+        projects = []
+
+        func = lambda o: o['project__id']
+        for project, entries in groupby(project_hours, func):
+            entries = list(entries)
+            proj_id = entries[0]['project__id']
+            name = entries[0]['project__name']
+            row = [None for i in range(len(id_list))]
+            for entry in entries:
+                index = id_list.index(entry['user__id'])
+                hours = entry['hours']
+                row[index] = row[index] + hours if row[index] else hours
+            projects.append((proj_id, name, row))
+
+        context.update({
+            'form': form,
+            'week': self.week_start,
+            'prev_week': self.week_start - relativedelta(days=7),
+            'next_week': self.week_start + relativedelta(days=7),
+            'people': people,
+            'project_hours': project_hours,
+            'projects': projects
+        })
+
+        return context
+
+
+class EditProjectHoursView(ProjectHoursMixin, TemplateView):
+    template_name = 'timepiece/hours/edit.html'
+    permissions = ('timepiece.add_projecthours',)
+
+    def get_context_data(self, **kwargs):
+        context = super(EditProjectHoursView, self).get_context_data(**kwargs)
+
+        form = timepiece_forms.ProjectHoursSearchForm(initial={
+            'week_start': self.week_start
+        })
+
+        context.update({
+            'form': form,
+            'week': self.week_start,
+            'ajax_url': reverse('project_hours_ajax_view')
+        })
+        return context
+
+    def post(self, request, *args, **kwargs):
+        ph = self.get_hours_for_week(self.week_start).filter(published=False)
+
+        if ph.exists():
+            ph.update(published=True)
+            msg = 'Unpublished project hours are now published'
+        else:
+            msg = 'There were no hours to publish'
+
+        messages.info(request, msg)
+
+        param = {
+            'week_start': self.week_start.strftime('%Y-%m-%d')
+        }
+        url = '?'.join((reverse('edit_project_hours'),
+            urllib.urlencode(param),))
+
+        return HttpResponseRedirect(url)
+
+
+class ProjectHoursAjaxView(ProjectHoursMixin, View):
+    permissions = ('timepiece.add_projecthours',)
+
+    def get_instance(self, data, week_start):
+        try:
+            user = auth_models.User.objects.get(pk=data.get('user', None))
+            project = timepiece.Project.objects.get(
+                pk=data.get('project', None))
+            hours = data.get('hours', None)
+            week = datetime.datetime.strptime(week_start, '%Y-%m-%d').date()
+
+            ph = timepiece.ProjectHours.objects.get(user=user, project=project,
+                week_start=week)
+            ph.hours = Decimal(hours)
+        except (exceptions.ObjectDoesNotExist):
+            ph = None
+
+        return ph
+
+    def get(self, request, *args, **kwargs):
+        """
+        Returns the data as a JSON object made up of the following key/value
+        pairs:
+            project_hours: the current project hours for the week
+            projects: the projects that have hours for the week
+            all_projects: all of the projects; used for autocomplete
+            all_users: all users that can clock in; used for completion
+        """
+        perm = auth_models.Permission.objects.filter(
+            content_type=ContentType.objects.get_for_model(timepiece.Entry),
+            codename='can_clock_in'
+        )
+        project_hours = self.get_hours_for_week().values(
+            'id', 'user', 'user__first_name', 'user__last_name',
+            'project', 'hours', 'published'
+        ).order_by('-project__type__billable', 'project__name',
+            'user__first_name', 'user__last_name')
+        inner_qs = project_hours.values_list('project', flat=True)
+        projects = timepiece.Project.objects.filter(pk__in=inner_qs).values() \
+            .order_by('name')
+        all_projects = timepiece.Project.objects.values('id', 'name')
+        all_users = auth_models.User.objects.filter(groups__permissions=perm) \
+            .values('id', 'first_name', 'last_name')
+
+        data = {
+            'project_hours': list(project_hours),
+            'projects': list(projects),
+            'all_projects': list(all_projects),
+            'all_users': list(all_users),
+            'ajax_url': reverse('project_hours_ajax_view'),
+        }
+        return HttpResponse(json.dumps(data, cls=DecimalEncoder),
+            mimetype='application/json')
+
+    def duplicate_entries(self, duplicate, week_update):
+        def duplicate_builder(queryset):
+            for instance in queryset:
+                duplicate = deepcopy(instance)
+                duplicate.id = None
+                duplicate.published = False
+                duplicate.week_start += datetime.timedelta(days=7)
+                yield duplicate
+
+        def duplicate_helper():
+            try:
+                try:
+                    bulk_create = getattr(timepiece.ProjectHours.objects,
+                        'bulk_create')
+                    bulk_create(duplicate_builder(prev_week_qs))
+                except AttributeError:
+                    for entry in duplicate_builder(prev_week_qs):
+                        entry.save()
+            except DatabaseError:
+                msg = 'An error occurred and hours could not be duplicated'
+                messages.error(self.request, msg)
+            else:
+                msg = 'Project hours were copied'
+                messages.info(self.request, msg)
+
+        date = datetime.datetime.strptime(week_update, '%Y-%m-%d').date()
+        prev_week = date - relativedelta(days=7)
+        prev_week_qs = self.get_hours_for_week(prev_week)
+        week_qs = self.get_hours_for_week(date)
+
+        param = {
+            'week_start': week_update
+        }
+        url = '?'.join((reverse('edit_project_hours'),
+            urllib.urlencode(param),))
+
+        if week_qs.exists():
+            inner_qs = week_qs.filter(
+                project__in=prev_week_qs.values_list('project'),
+                user__in=prev_week_qs.values_list('user'))
+
+            if inner_qs.exists():
+                for ph in inner_qs:
+                    prev_ph = prev_week_qs.get(project=ph.project,
+                        user=ph.user)
+                    ph.hours = prev_ph.hours
+                    ph.published = False
+                    ph.save()
+                msg = 'Project hours were copied'
+                messages.info(self.request, msg)
+            else:
+                duplicate_helper()
+        elif not prev_week_qs.exists():
+            msg = 'There are no hours to copy'
+            messages.warning(self.request, msg)
+        else:
+            duplicate_helper()
+
+        return HttpResponseRedirect(url)
+
+    def update_week(self, week_start):
+        try:
+            instance = self.get_instance(self.request.POST, week_start)
+        except TypeError:
+            msg = 'Parameter week_start must be a date in the format ' \
+                'yyyy-mm-dd'
+            return HttpResponse(msg, status=500)
+
+        form = timepiece_forms.ProjectHoursForm(self.request.POST,
+            instance=instance)
+
+        if form.is_valid():
+            ph = form.save()
+            return HttpResponse(str(ph.pk), mimetype='text/plain')
+
+        msg = 'The request must contain values for user, project, and hours'
+        return HttpResponse(msg, status=500)
+
+    def post(self, request, *args, **kwargs):
+        """
+        Create or update an hour entry for a particular use and project. This
+        function expects the following values:
+            user: the user pk for the hours
+            project: the project pk for the hours
+            hours: the actual hours to store
+            week_start: the start of the week for the hours
+
+        If the duplicate key is present along with week_update, then items
+        will be duplicated from week_update to the current week
+        """
+        duplicate = request.POST.get('duplicate', None)
+        week_update = request.POST.get('week_update', None)
+        week_start = request.POST.get('week_start', None)
+
+        if duplicate and week_update:
+            return self.duplicate_entries(duplicate, week_update)
+
+        return self.update_week(week_start)
+
+
+class ProjectHoursDetailView(ProjectHoursMixin, View):
+    permissions = ('timepiece.add_projecthours',)
+
+    def delete(self, request, *args, **kwargs):
+        """
+        Remove a project from the database
+        """
+        pk = kwargs.get('pk', None)
+
+        if pk:
+            try:
+                ph = timepiece.ProjectHours.objects.get(pk=pk)
+            except timepiece.ProjectHours.DoesNotExist:
+                pass
+            else:
+                ph.delete()
+                return HttpResponse('ok', mimetype='text/plain')
+
+        return HttpResponse('', status=500)
